using System;
using System.Diagnostics;
using Zcu.Mve.Core;


namespace Zcu.Mve.Numerics
{
	/// <summary>
	/// Summary description for Matrix.
	/// </summary>
	public class Matrix:IDataObject
	{
		private int width, height;
		private double[] matrix;

        /// <summary>
        /// Width of the matrix. Read-only.
        /// </summary>
		public int Width
		{
			get{return(width);}
		}

        /// <summary>
        /// Height of the matrix. Read-only.
        /// </summary>
		public int Height
		{
			get{return(height);}
		}

        /// <summary>
        /// Default constructor, constructs a new Double matrix of width w and heigh h.
        /// </summary>
        /// <param name="w">Width of the new matrix.</param>
        /// <param name="h">Height of the new matrix.</param>
		public Matrix(int w, int h)
		{
			this.width = w;
			this.height = h;
			this.matrix = new double[h*w];
		}

        /// <summary>
        /// Public indexer that returns matrix value at given row and column. 
        /// No consistency checks are performed, may throw exception when wrong indices are passed.
        /// </summary>
        /// <param name="row">Zero based row index.</param>
        /// <param name="column">Zero based column index.</param>
        /// <returns>Double value at given indices.</returns>
		public double this[int row,int column]
		{
			get
			{
				return(this.matrix[row*width + column]);
			}
			set
			{
				this.matrix[row*width + column] = value;
			}
		}

		#region IDataObject Members

        /// <summary>
        /// Writes data to given xmlTextWriter.
        /// </summary>
        /// <param name="xmlTextWriter">A xmlTextWriter instance to be used.</param>
		public void WriteData(System.Xml.XmlTextWriter xmlTextWriter)
		{
			xmlTextWriter.WriteElementString("Width",this.width.ToString(Globals.nfi));
			xmlTextWriter.WriteEndElement();
			xmlTextWriter.WriteElementString("Height",this.height.ToString(Globals.nfi));
			xmlTextWriter.WriteEndElement();
			for (int i = 0;i<this.height;i++)
			{
				string row = "";
				for (int j = 0;j<this.width;j++)
				{
					row+=this.matrix[i*this.width+j].ToString(Globals.nfi)+" ";
				}
				xmlTextWriter.WriteElementString("Row",row);
				xmlTextWriter.WriteEndElement();
			}
		}

        /// <summary>
        /// Returns anew instance of Matrix that contains a copy of the current matrix.
        /// </summary>
        /// <returns>A deep copy of the matrix.</returns>
		public IDataObject DeepCopy()
		{
			Matrix result = new Matrix(this.width, this.height);
			Array.Copy(this.matrix,result.matrix,this.matrix.Length);
			return(result);
		}

        /// <summary>
        /// Reads data from xmlTextReader. Data format should match the format used by the WriteData(...) method.
        /// </summary>
        /// <param name="xmlTextReader">A xmlTextReader instance to read from.</param>
		public void ReadData(System.Xml.XmlTextReader xmlTextReader)
		{
			string w = xmlTextReader.ReadElementString("Width");
			this.width = int.Parse(w,Globals.nfi);
			xmlTextReader.ReadEndElement();

			string h = xmlTextReader.ReadElementString("Height");
			this.height = int.Parse(h,Globals.nfi);
			xmlTextReader.ReadEndElement();

			this.matrix = new double[this.height*this.width];

			for (int i = 0;i<this.height;i++)
			{
				string row = xmlTextReader.ReadElementString("Row");
				xmlTextReader.ReadEndElement();

				string[] columns = row.Split(new char[]{' '},this.width);
				for (int j = 0;j<this.width;j++)
					this.matrix[i*width+j] = double.Parse(columns[j],Globals.nfi);
			}
		}

        /// <summary>
        /// Returns a list of libraries necessary for the type.
        /// </summary>
		public string[] LibNames
		{
			get
			{
				return new string [] {Globals.GetLibraryName(this.GetType())};
			}
		}

        /// <summary>
        /// Checks consistence of the matrix instance. Always returns true, but may also write warnings when matrix contains infifnities of NaNs.
        /// </summary>
        /// <returns>Always true.</returns>
		public bool CheckConsistence()
		{
			for (int i = 0;i<this.matrix.Length;i++)
			{
				if (double.IsInfinity(this.matrix[i])) 
					Debug.WriteLine("Matrix contains infinity value(s).");
				if (double.IsNaN(this.matrix[i]))
					Debug.WriteLine("Matrix contains NaN value(s).");
			}
			return true;
		}

		#endregion
	}
}
