using System;
using Zcu.Mve.Core;

namespace Zcu.Mve.Numerics
{
	/// <summary>
	/// This structure encapsulates a single integer value and implements the IDataObject interface.
	/// </summary>
	[DataObject("This structure encapsulates a single integer value and implements the IDataObject interface.")]
	public struct MveInt:IDataObject
	{
        /// <summary>
        /// wrapped integer value
        /// </summary>
		public int Value;


        /// <summary>
        /// Constructs a new instance of MveInt, that will wrap the value passed as parameter.
        /// </summary>
        /// <param name="val">Value to be wrapped by the new instance.</param>
		public MveInt(int val)
		{
			this.Value = val;
		}

		/// <summary>
		/// Read data from Xml file to current structure.
		/// </summary>
		/// <param name="xmlTextReader">Stream to be read.</param>
		public void ReadData(System.Xml.XmlTextReader xmlTextReader)
		{
			string tmp = xmlTextReader.ReadString().Trim();
			this.Value = int.Parse(tmp, Globals.nfi);
		} // ReadData()

		/// <summary>
		/// Write current data to Xml file.
		/// </summary>
		/// <param name="xmlTextWriter">Stream to be write.</param>
		public void WriteData(System.Xml.XmlTextWriter xmlTextWriter)
		{
			xmlTextWriter.WriteString(this.Value.ToString(Globals.nfi));
		} // WriteData()

		/// <summary>
		/// Deep copy of data object and its subelements.
		/// </summary>
		/// <returns>Same data, different memory space.</returns>
		public IDataObject DeepCopy()
		{
			return new MveInt(this.Value);
		} // DeepCopy()

		/// <summary>
		/// Check the consistency of current data structure.
		/// </summary>
		/// <returns>True if structure is consistent.</returns>
		public bool CheckConsistence()
		{
			return true;
		} // CheckConsistence()

		// ===== Methods of the class Object ==============================================
		#region Overriden methods of the class Object

		/// <summary>
		/// Returns a String that represents the current Object.
		/// </summary>
		/// <returns>String that represents the current Object.</returns>
		public override string ToString()
		{
			return "MveInt: " + this.Value;
		} // ToString()

		/// <summary>
		/// Determines whether the specified Objects are equal.
		/// </summary>
		/// <param name="a">First Object.</param>
		/// <param name="b">Second Object.</param>
		/// <returns>True if objects are equal. Otherwise, false.</returns>
		public static new bool Equals(object a, object b)
		{
			if ((a is MveInt) && (b is MveInt))
			{
				return(((MveInt)a).Value == ((MveInt)b).Value);
			}
			return false;
		}

		/// <summary>
		/// Determines whether the specified Object is equal to the current Object.
		/// </summary>
		/// <param name="obj">Specified Object to compare with the current Object.</param>
		/// <returns>True if the specified Object is equal to the current Object. Otherwise, false.</returns>
		public override bool Equals(object obj)
		{
			return MveInt.Equals(this, obj);
		}

		/// <summary>
		/// Returns a hash code for the current Object.
		/// </summary>
		/// <returns>A hash code for the current Object.</returns>
		public override int GetHashCode()
		{
			return this.Value;
		}

		/// <summary>
		/// Returns array of assembly name.
		/// </summary>
		public string [] LibNames
		{
			get
			{
				return new string [] {Globals.GetLibraryName(this.GetType())};
			}
		}

		#endregion

		// ===== Conversions ==============================================================
		#region Conversions

		// ===== MveInt <-> int ================
		/// <summary>
		/// Implicit conversion MveInt -> int.
		/// </summary>
		/// <param name="i">MveInt.</param>
		/// <returns>int.</returns>
		public static implicit operator int(MveInt i) 
		{
			return i.Value;
		} // MveInt -> int

		/// <summary>
		/// Implicit conversion int -> MveInt
		/// </summary>
		/// <param name="i">int.</param>
		/// <returns>MveInt.</returns>
		public static implicit operator MveInt(int i)
		{
			return new MveInt(i);
		} // int -> MveInt

		#endregion

		// ===== Operators ================================================================
		#region Operators

		/// <summary>
		/// The equality operator (==) returns true if the values of its operands are equal, false otherwise.
		/// </summary>
		/// <param name="a">First operand.</param>
		/// <param name="b">Second operand.</param>
		/// <returns>True if the values of operands are equal, false otherwise.</returns>
		public static bool operator ==(MveInt a, MveInt b)
		{
			return MveInt.Equals(a, b);
		}

		/// <summary>
		/// The inequality operator (!=) returns false if its operands are equal, true otherwise.
		/// </summary>
		/// <param name="a">First operand.</param>
		/// <param name="b">Second operand.</param>
		/// <returns>False if the values of operands are equal, true otherwise.</returns>
		public static bool operator !=(MveInt a, MveInt b) 
		{
			return !(a == b);
		}

        /// <summary>
        /// Increment operator, returns a new MveInt that wraps a value incremented by 1.
        /// </summary>
        /// <param name="a">Value to be inceremented.</param>
        /// <returns>Incremented value.</returns>
		public static MveInt operator ++(MveInt a)
		{
			return new MveInt(a.Value+1);
		}

        /// <summary>
        /// Decrement operator, returns a new MveInt that wraps a value decremented by 1.
        /// </summary>
        /// <param name="a">A value to be decremented.</param>
        /// <returns>Decremented value.</returns>
		public static MveInt operator --(MveInt a)
		{
			return new MveInt(a.Value-1);
		}

		#endregion
	}
}
